@@ -184,15 +184,27 @@ static void pgsql_reconnect(void)
 		conn = NULL;
 	}
 
-	ast_str_set(&conn_info, 0, "host=%s port=%s dbname=%s user=%s",
-		pghostname, pgdbport, pgdbname, pgdbuser);
-
+	if (!ast_strlen_zero(pghostname)) {
+		ast_str_append(&conn_info, 0, "host=%s ", pghostname);
+	}
+	if (!ast_strlen_zero(pgdbport)) {
+		ast_str_append(&conn_info, 0, "port=%s ", pgdbport);
+	}
+	if (!ast_strlen_zero(pgdbname)) {
+		ast_str_append(&conn_info, 0, "dbname=%s ", pgdbname);
+	}
+	if (!ast_strlen_zero(pgdbuser)) {
+		ast_str_append(&conn_info, 0, "user=%s ", pgdbuser);
+	}
 	if (!ast_strlen_zero(pgappname)) {
-		ast_str_append(&conn_info, 0, " application_name=%s", pgappname);
+		ast_str_append(&conn_info, 0, "application_name=%s ", pgappname);
 	}
-
 	if (!ast_strlen_zero(pgpassword)) {
-		ast_str_append(&conn_info, 0, " password=%s", pgpassword);
+		ast_str_append(&conn_info, 0, "password=%s", pgpassword);
+	}
+	if (ast_str_strlen(conn_info) == 0) {
+		ast_log(LOG_ERROR, "Connection string is blank.\n");
+		return;
 	}
 
 	conn = PQconnectdb(ast_str_buffer(conn_info));
@@ -695,7 +707,7 @@ static int config_module(int reload)
 			schemaname = ast_alloca(strlen(tmp_schemaname) * 2 + 1);
 			PQescapeStringConn(conn, schemaname, tmp_schemaname, strlen(tmp_schemaname), NULL);
 
-			snprintf(sqlcmd, sizeof(sqlcmd), "SELECT a.attname, t.typname, a.attlen, a.attnotnull, d.adsrc, a.atttypmod FROM (((pg_catalog.pg_class c INNER JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace AND c.relname = '%s' AND n.nspname = %s%s%s) INNER JOIN pg_catalog.pg_attribute a ON (NOT a.attisdropped) AND a.attnum > 0 AND a.attrelid = c.oid) INNER JOIN pg_catalog.pg_type t ON t.oid = a.atttypid) LEFT OUTER JOIN pg_attrdef d ON a.atthasdef AND d.adrelid = a.attrelid AND d.adnum = a.attnum ORDER BY n.nspname, c.relname, attnum",
+			snprintf(sqlcmd, sizeof(sqlcmd), "SELECT a.attname, t.typname, a.attlen, a.attnotnull, pg_catalog.pg_get_expr(d.adbin, d.adrelid) adsrc, a.atttypmod FROM (((pg_catalog.pg_class c INNER JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace AND c.relname = '%s' AND n.nspname = %s%s%s) INNER JOIN pg_catalog.pg_attribute a ON (NOT a.attisdropped) AND a.attnum > 0 AND a.attrelid = c.oid) INNER JOIN pg_catalog.pg_type t ON t.oid = a.atttypid) LEFT OUTER JOIN pg_attrdef d ON a.atthasdef AND d.adrelid = a.attrelid AND d.adnum = a.attnum ORDER BY n.nspname, c.relname, attnum",
 				tablename,
 				ast_strlen_zero(schemaname) ? "" : "'", ast_strlen_zero(schemaname) ? "current_schema()" : schemaname, ast_strlen_zero(schemaname) ? "" : "'");
 		} else {
