@@ -1358,5 +1358,23 @@ ALTER TABLE ps_systems ADD COLUMN disable_rport ast_bool_values;
 
 UPDATE alembic_version SET version_num='79290b511e4b' WHERE alembic_version.version_num = 'fbb7766f17bc';
 
+-- Running upgrade 79290b511e4b -> 1ae0609b6646
+
+ALTER TABLE ps_contacts ALTER COLUMN reg_server TYPE VARCHAR(255);
+
+UPDATE alembic_version SET version_num='1ae0609b6646' WHERE alembic_version.version_num = '79290b511e4b';
+
+-- Running upgrade 1ae0609b6646 -> e658c26033ca
+
+ALTER TABLE ps_endpoints ADD COLUMN send_history_info ast_bool_values;
+
+UPDATE alembic_version SET version_num='e658c26033ca' WHERE alembic_version.version_num = '1ae0609b6646';
+
+-- Running upgrade e658c26033ca -> 61797b9fced6
+
+ALTER TABLE ps_endpoints ADD COLUMN stir_shaken ast_bool_values;
+
+UPDATE alembic_version SET version_num='61797b9fced6' WHERE alembic_version.version_num = 'e658c26033ca';
+
 COMMIT;
 
