@@ -173,6 +173,11 @@ extern struct ast_format *ast_format_h263p;
  */
 extern struct ast_format *ast_format_h264;
 
+/*!
+ * \brief Built-in cached h265 format.
+ */
+extern struct ast_format *ast_format_h265;
+
 /*!
  * \brief Built-in cached mp4 format.
  */
@@ -309,4 +314,17 @@ struct ast_format *ast_format_cache_get_slin_by_rate(unsigned int rate);
  */
 int ast_format_cache_is_slinear(struct ast_format *format);
 
+/*!
+ * \brief Retrieve a format from the cache by its codec
+ *
+ * \param codec The codec to search by
+ *
+ * \retval non-NULL if found
+ * \retval NULL if not found
+ *
+ * \note The returned format has its reference count incremented. It must be
+ * dropped using ao2_ref or ao2_cleanup.
+ */
+struct ast_format *ast_format_cache_get_by_codec(const struct ast_codec *codec);
+
 #endif /* _AST_FORMAT_CACHE_H */
