@@ -629,12 +629,21 @@ struct ast_json *ast_json_name_number(const char *name, const char *number)
 		"number", AST_JSON_UTF8_VALIDATE(number));
 }
 
+struct ast_json *ast_json_dialplan_cep_app(
+		const char *context, const char *exten, int priority, const char *app_name, const char *app_data)
+{
+	return ast_json_pack("{s: s?, s: s?, s: o, s: s?, s: s?}",
+		"context", context,
+		"exten", exten,
+		"priority", priority != -1 ? ast_json_integer_create(priority) : ast_json_null(),
+		"app_name", app_name,
+		"app_data", app_data
+		);
+}
+
 struct ast_json *ast_json_dialplan_cep(const char *context, const char *exten, int priority)
 {
-	return ast_json_pack("{s: o, s: o, s: o}",
-		"context", context ? ast_json_string_create(context) : ast_json_null(),
-		"exten", exten ? ast_json_string_create(exten) : ast_json_null(),
-		"priority", priority != -1 ? ast_json_integer_create(priority) : ast_json_null());
+	return ast_json_dialplan_cep_app(context, exten, priority, "", "");
 }
 
 struct ast_json *ast_json_timeval(const struct timeval tv, const char *zone)
@@ -693,10 +702,28 @@ struct ast_json *ast_json_ipaddr(const struct ast_sockaddr *addr, enum ast_trans
 	return ast_json_string_create(ast_str_buffer(string));
 }
 
-void ast_json_init(void)
+int ast_json_init(void)
 {
+	json_t *version_check;
+
 	/* Setup to use Asterisk custom allocators */
 	ast_json_reset_alloc_funcs();
+
+	/* We depend on functionality of jansson-2.11 but don't actually use
+	 * any symbols.  If we link at runtime to less than 2.11 this json_pack
+	 * will return NULL. */
+	version_check = json_pack("{s: o?, s: o*}",
+		"JSON", NULL,
+		"Bourne", NULL);
+	if (!version_check) {
+		ast_log(LOG_ERROR, "There was a problem finding jansson 2.11 runtime libraries.\n"
+			"Please rebuild Asterisk using ./configure --with-jansson-bundled.\n");
+		return -1;
+	}
+
+	json_decref(version_check);
+
+	return 0;
 }
 
 static void json_payload_destructor(void *obj)
