@@ -98,6 +98,7 @@ char record_cache_dir[AST_CACHE_DIR_LEN] = DEFAULT_TMP_DIR;
 char ast_defaultlanguage[MAX_LANGUAGE] = DEFAULT_LANGUAGE;
 
 struct _cfg_paths {
+	char cache_dir[PATH_MAX];
 	char config_dir[PATH_MAX];
 	char module_dir[PATH_MAX];
 	char spool_dir[PATH_MAX];
@@ -125,6 +126,7 @@ struct _cfg_paths {
 };
 
 static struct _cfg_paths cfg_paths = {
+	.cache_dir = DEFAULT_CACHE_DIR,
 	.config_dir = DEFAULT_CONFIG_DIR,
 	.module_dir = DEFAULT_MODULE_DIR,
 	.spool_dir = DEFAULT_SPOOL_DIR,
@@ -145,6 +147,7 @@ static struct _cfg_paths cfg_paths = {
 	.ctl_file = "asterisk.ctl",
 };
 
+const char *ast_config_AST_CACHE_DIR	= cfg_paths.cache_dir;
 const char *ast_config_AST_CONFIG_DIR	= cfg_paths.config_dir;
 const char *ast_config_AST_CONFIG_FILE	= cfg_paths.config_file;
 const char *ast_config_AST_MODULE_DIR	= cfg_paths.module_dir;
@@ -254,7 +257,9 @@ void load_asterisk_conf(void)
 	}
 
 	for (v = ast_variable_browse(cfg, "directories"); v; v = v->next) {
-		if (!strcasecmp(v->name, "astetcdir")) {
+		if (!strcasecmp(v->name, "astcachedir")) {
+			ast_copy_string(cfg_paths.cache_dir, v->value, sizeof(cfg_paths.cache_dir));
+		} else if (!strcasecmp(v->name, "astetcdir")) {
 			ast_copy_string(cfg_paths.config_dir, v->value, sizeof(cfg_paths.config_dir));
 		} else if (!strcasecmp(v->name, "astspooldir")) {
 			ast_copy_string(cfg_paths.spool_dir, v->value, sizeof(cfg_paths.spool_dir));
