@@ -990,8 +990,9 @@ struct ast_json *ast_channel_snapshot_to_json(
 			snapshot->connected_name, snapshot->connected_number),
 		"accountcode", snapshot->accountcode,
 		/* Third line */
-		"dialplan", ast_json_dialplan_cep(
-			snapshot->context, snapshot->exten, snapshot->priority),
+		"dialplan", ast_json_dialplan_cep_app(
+			snapshot->context, snapshot->exten, snapshot->priority,
+			snapshot->appl, snapshot->data),
 		"creationtime", ast_json_timeval(snapshot->creationtime, NULL),
 		"language", snapshot->language);
 
@@ -1116,11 +1117,11 @@ static struct ast_json *dtmf_end_to_json(
 		return NULL;
 	}
 
-	return ast_json_pack("{s: s, s: o, s: s, s: i, s: o}",
+	return ast_json_pack("{s: s, s: o, s: s, s: I, s: o}",
 		"type", "ChannelDtmfReceived",
 		"timestamp", ast_json_timeval(*tv, NULL),
 		"digit", digit,
-		"duration_ms", duration_ms,
+		"duration_ms", (ast_json_int_t)duration_ms,
 		"channel", json_channel);
 }
 
@@ -1295,6 +1296,7 @@ STASIS_MESSAGE_TYPE_DEFN(ast_channel_varset_type,
 STASIS_MESSAGE_TYPE_DEFN(ast_channel_hangup_request_type,
 	.to_json = hangup_request_to_json,
 	);
+STASIS_MESSAGE_TYPE_DEFN(ast_channel_masquerade_type);
 STASIS_MESSAGE_TYPE_DEFN(ast_channel_dtmf_begin_type);
 STASIS_MESSAGE_TYPE_DEFN(ast_channel_dtmf_end_type,
 	.to_json = dtmf_end_to_json,
@@ -1343,6 +1345,7 @@ static void stasis_channels_cleanup(void)
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_dial_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_varset_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_hangup_request_type);
+	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_masquerade_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_dtmf_begin_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_dtmf_end_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_hold_type);
@@ -1367,7 +1370,7 @@ int ast_stasis_channels_init(void)
 
 	ast_register_cleanup(stasis_channels_cleanup);
 
-	channel_cache_all = stasis_cp_all_create("ast_channel_topic_all",
+	channel_cache_all = stasis_cp_all_create("channel:all",
 		channel_snapshot_get_id);
 	if (!channel_cache_all) {
 		return -1;
@@ -1393,6 +1396,7 @@ int ast_stasis_channels_init(void)
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_dial_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_varset_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_hangup_request_type);
+	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_masquerade_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_dtmf_begin_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_dtmf_end_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_hold_type);
