@@ -111,14 +111,13 @@ static pjsip_transport *get_udp_transport(pj_str_t *address, int port)
 	}
 
 	for (iter = ao2_iterator_init(transport_states, 0); (transport_state = ao2_iterator_next(&iter)); ao2_ref(transport_state, -1)) {
-		if (transport_state && ((transport_state->type != AST_TRANSPORT_UDP) ||
-			(pj_strcmp(&transport_state->transport->local_name.host, address)) ||
-			(transport_state->transport->local_name.port != port))) {
-			continue;
+		if (transport_state->type == AST_TRANSPORT_UDP &&
+			!pj_strcmp(&transport_state->transport->local_name.host, address) &&
+			transport_state->transport->local_name.port == port) {
+			sip_transport = transport_state->transport;
+			ao2_ref(transport_state, -1);
+			break;
 		}
-
-		sip_transport = transport_state->transport;
-		break;
 	}
 	ao2_iterator_destroy(&iter);
 
@@ -237,7 +236,7 @@ static pj_status_t filter_on_tx_message(pjsip_tx_data *tdata)
 	pjsip_tpmgr_fla2_param_default(&prm);
 	prm.tp_type = tdata->tp_info.transport->key.type;
 	pj_strset2(&prm.dst_host, tdata->tp_info.dst_name);
-	prm.local_if = is_bound_any(tdata->tp_info.transport);
+	prm.local_if = PJ_TRUE;
 
 	/* If we can't get the local address use best effort and let it pass */
 	if (pjsip_tpmgr_find_local_addr2(pjsip_endpt_get_tpmgr(ast_sip_get_pjsip_endpoint()), tdata->pool, &prm) != PJ_SUCCESS) {
@@ -401,6 +400,40 @@ static void print_uri_debug(enum uri_type ut, pjsip_rx_data *rdata, pjsip_hdr *h
 #endif
 }
 
+/*!
+ * /internal
+ *
+ * We want to make sure that any incoming requests don't already
+ * have x-ast-* parameters in any URIs or we may get confused
+ * if symmetric transport (x-ast-txp) or rewrite_contact (x-ast-orig-host)
+ * is used later on.
+ */
+static void remove_x_ast_params(pjsip_uri *header_uri){
+	pjsip_sip_uri *uri;
+	pjsip_param *param;
+
+	if (!header_uri) {
+		return;
+	}
+
+	uri = pjsip_uri_get_uri(header_uri);
+	if (!uri) {
+		return;
+	}
+
+	param = uri->other_param.next;
+
+	while (param != &uri->other_param) {
+		/* We need to save off 'next' because pj_list_erase will remove it */
+		pjsip_param *next = param->next;
+
+		if (pj_strncmp2(&param->name, "x-ast-", 6) == 0) {
+			pj_list_erase(param);
+		}
+		param = next;
+	}
+}
+
 static pj_bool_t on_rx_process_uris(pjsip_rx_data *rdata)
 {
 	pjsip_contact_hdr *contact = NULL;
@@ -415,6 +448,7 @@ static pj_bool_t on_rx_process_uris(pjsip_rx_data *rdata)
 			PJSIP_SC_UNSUPPORTED_URI_SCHEME, NULL, NULL, NULL);
 		return PJ_TRUE;
 	}
+	remove_x_ast_params(rdata->msg_info.msg->line.req.uri);
 
 	if (!is_sip_uri(rdata->msg_info.from->uri)) {
 		print_uri_debug(URI_TYPE_FROM, rdata, (pjsip_hdr *)rdata->msg_info.from);
@@ -422,6 +456,7 @@ static pj_bool_t on_rx_process_uris(pjsip_rx_data *rdata)
 			PJSIP_SC_UNSUPPORTED_URI_SCHEME, NULL, NULL, NULL);
 		return PJ_TRUE;
 	}
+	remove_x_ast_params(rdata->msg_info.from->uri);
 
 	if (!is_sip_uri(rdata->msg_info.to->uri)) {
 		print_uri_debug(URI_TYPE_TO, rdata, (pjsip_hdr *)rdata->msg_info.to);
@@ -429,7 +464,7 @@ static pj_bool_t on_rx_process_uris(pjsip_rx_data *rdata)
 			PJSIP_SC_UNSUPPORTED_URI_SCHEME, NULL, NULL, NULL);
 		return PJ_TRUE;
 	}
-
+	remove_x_ast_params(rdata->msg_info.to->uri);
 
 	contact = (pjsip_contact_hdr *) pjsip_msg_find_hdr(
 		rdata->msg_info.msg, PJSIP_H_CONTACT, NULL);
@@ -449,6 +484,8 @@ static pj_bool_t on_rx_process_uris(pjsip_rx_data *rdata)
 				PJSIP_SC_UNSUPPORTED_URI_SCHEME, NULL, NULL, NULL);
 			return PJ_TRUE;
 		}
+		remove_x_ast_params(contact->uri);
+
 		contact = (pjsip_contact_hdr *) pjsip_msg_find_hdr(
 			rdata->msg_info.msg, PJSIP_H_CONTACT, contact->next);
 	}
